@@ -24,9 +24,22 @@
 
 #include <stdint.h>
 
+#include "gpio.h"
+
+typedef struct jtag_gpio_t {
+	gpio_t gpio_tms;
+	gpio_t gpio_tck;
+	gpio_t gpio_tdi;
+	gpio_t gpio_tdo;
+} jtag_gpio_t;
+
+typedef struct jtag_t {
+	jtag_gpio_t* const gpio;
+} jtag_t;
+
 typedef void (*refill_buffer_cb)(void);
 
-void cpld_jtag_release(void);
+void cpld_jtag_release(jtag_t* const jtag);
 
 /* Return 0 if success else return error code see xsvfExecute() see micro.h.
  *
@@ -34,6 +47,7 @@ void cpld_jtag_release(void);
  * contents of the buffer has been streamed to the CPLD the given
  * refill_buffer callback will be called. */
 int cpld_jtag_program(
+		jtag_t* const jtag,
         const uint32_t buffer_length,
         unsigned char* const buffer,
         refill_buffer_cb refill
