@@ -22,7 +22,6 @@
 
 #include "rf_path.h"
 
-#include <libopencm3/lpc43xx/gpio.h>
 #include <libopencm3/lpc43xx/scu.h>
 
 #include <hackrf_core.h>
@@ -84,58 +83,58 @@ uint8_t switchctrl = SWITCHCTRL_SAFE;
 #define SWITCHCTRL_ANT_PWR (1 << 6) /* turn on antenna port power */
 
 #ifdef HACKRF_ONE
-static void switchctrl_set_hackrf_one(uint8_t ctrl) {
+static void switchctrl_set_hackrf_one(rf_path_t* const rf_path, uint8_t ctrl) {
 	if (ctrl & SWITCHCTRL_TX) {
-		gpio_set(PORT_TX, PIN_TX);
-		gpio_clear(PORT_RX, PIN_RX);
+		gpio_set(rf_path->gpio_tx);
+		gpio_clear(rf_path->gpio_rx);
 	} else {
-		gpio_clear(PORT_TX, PIN_TX);
-		gpio_set(PORT_RX, PIN_RX);
+		gpio_clear(rf_path->gpio_tx);
+		gpio_set(rf_path->gpio_rx);
 	}
 
 	if (ctrl & SWITCHCTRL_MIX_BYPASS) {
-		gpio_set(PORT_MIX_BYPASS, PIN_MIX_BYPASS);
-		gpio_clear(PORT_NO_MIX_BYPASS, PIN_NO_MIX_BYPASS);
+		gpio_set(rf_path->gpio_mix_bypass);
+		gpio_clear(rf_path->gpio_no_mix_bypass);
 		if (ctrl & SWITCHCTRL_TX) {
-			gpio_set(PORT_TX_MIX_BP, PIN_TX_MIX_BP);
-			gpio_clear(PORT_RX_MIX_BP, PIN_RX_MIX_BP);
+			gpio_set(rf_path->gpio_tx_mix_bp);
+			gpio_clear(rf_path->gpio_rx_mix_bp);
 		} else {
-			gpio_clear(PORT_TX_MIX_BP, PIN_TX_MIX_BP);
-			gpio_set(PORT_RX_MIX_BP, PIN_RX_MIX_BP);
+			gpio_clear(rf_path->gpio_tx_mix_bp);
+			gpio_set(rf_path->gpio_rx_mix_bp);
 		}
 	} else {
-		gpio_clear(PORT_MIX_BYPASS, PIN_MIX_BYPASS);
-		gpio_set(PORT_NO_MIX_BYPASS, PIN_NO_MIX_BYPASS);
-		gpio_clear(PORT_TX_MIX_BP, PIN_TX_MIX_BP);
-		gpio_clear(PORT_RX_MIX_BP, PIN_RX_MIX_BP);
+		gpio_clear(rf_path->gpio_mix_bypass);
+		gpio_set(rf_path->gpio_no_mix_bypass);
+		gpio_clear(rf_path->gpio_tx_mix_bp);
+		gpio_clear(rf_path->gpio_rx_mix_bp);
 	}
 
 	if (ctrl & SWITCHCTRL_HP) {
-		gpio_set(PORT_HP, PIN_HP);
-		gpio_clear(PORT_LP, PIN_LP);
+		gpio_set(rf_path->gpio_hp);
+		gpio_clear(rf_path->gpio_lp);
 	} else {
-		gpio_clear(PORT_HP, PIN_HP);
-		gpio_set(PORT_LP, PIN_LP);
+		gpio_clear(rf_path->gpio_hp);
+		gpio_set(rf_path->gpio_lp);
 	}
 
 	if (ctrl & SWITCHCTRL_AMP_BYPASS) {
-		gpio_set(PORT_AMP_BYPASS, PIN_AMP_BYPASS);
-		gpio_clear(PORT_TX_AMP, PIN_TX_AMP);
-		gpio_set(PORT_NO_TX_AMP_PWR, PIN_NO_TX_AMP_PWR);
-		gpio_clear(PORT_RX_AMP, PIN_RX_AMP);
-		gpio_set(PORT_NO_RX_AMP_PWR, PIN_NO_RX_AMP_PWR);
+		gpio_set(rf_path->gpio_amp_bypass);
+		gpio_clear(rf_path->gpio_tx_amp);
+		gpio_set(rf_path->gpio_no_tx_amp_pwr);
+		gpio_clear(rf_path->gpio_rx_amp);
+		gpio_set(rf_path->gpio_no_rx_amp_pwr);
 	} else if (ctrl & SWITCHCTRL_TX) {
-		gpio_clear(PORT_AMP_BYPASS, PIN_AMP_BYPASS);
-		gpio_set(PORT_TX_AMP, PIN_TX_AMP);
-		gpio_clear(PORT_NO_TX_AMP_PWR, PIN_NO_TX_AMP_PWR);
-		gpio_clear(PORT_RX_AMP, PIN_RX_AMP);
-		gpio_set(PORT_NO_RX_AMP_PWR, PIN_NO_RX_AMP_PWR);
+		gpio_clear(rf_path->gpio_amp_bypass);
+		gpio_set(rf_path->gpio_tx_amp);
+		gpio_clear(rf_path->gpio_no_tx_amp_pwr);
+		gpio_clear(rf_path->gpio_rx_amp);
+		gpio_set(rf_path->gpio_no_rx_amp_pwr);
 	} else {
-		gpio_clear(PORT_AMP_BYPASS, PIN_AMP_BYPASS);
-		gpio_clear(PORT_TX_AMP, PIN_TX_AMP);
-		gpio_set(PORT_NO_TX_AMP_PWR, PIN_NO_TX_AMP_PWR);
-		gpio_set(PORT_RX_AMP, PIN_RX_AMP);
-		gpio_clear(PORT_NO_RX_AMP_PWR, PIN_NO_RX_AMP_PWR);
+		gpio_clear(rf_path->gpio_amp_bypass);
+		gpio_clear(rf_path->gpio_tx_amp);
+		gpio_set(rf_path->gpio_no_tx_amp_pwr);
+		gpio_set(rf_path->gpio_rx_amp);
+		gpio_clear(rf_path->gpio_no_rx_amp_pwr);
 	}
 
 	/*
@@ -144,29 +143,30 @@ static void switchctrl_set_hackrf_one(uint8_t ctrl) {
 	 * is unset:
 	 */
 	if (ctrl & SWITCHCTRL_NO_TX_AMP_PWR)
-		gpio_set(PORT_NO_TX_AMP_PWR, PIN_NO_TX_AMP_PWR);
+		gpio_set(rf_path->gpio_no_tx_amp_pwr);
 	if (ctrl & SWITCHCTRL_NO_RX_AMP_PWR)
-		gpio_set(PORT_NO_RX_AMP_PWR, PIN_NO_RX_AMP_PWR);
+		gpio_set(rf_path->gpio_no_rx_amp_pwr);
 
 	if (ctrl & SWITCHCTRL_ANT_PWR) {
-		rffc5071_set_gpo(0x00); /* turn on antenna power by clearing GPO1 */
+		rffc5071_set_gpo(&rffc5072, 0x00); /* turn on antenna power by clearing GPO1 */
 	} else {
-		rffc5071_set_gpo(0x01); /* turn off antenna power by setting GPO1 */
+		rffc5071_set_gpo(&rffc5072, 0x01); /* turn off antenna power by setting GPO1 */
 	}
 }
 #endif
 
-static void switchctrl_set(const uint8_t gpo) {
+static void switchctrl_set(rf_path_t* const rf_path, const uint8_t gpo) {
 #ifdef JAWBREAKER
-	rffc5071_set_gpo(gpo);
+	(void) rf_path; /* silence unused param warning */
+	rffc5071_set_gpo(&rffc5072, gpo);
 #elif HACKRF_ONE
-	switchctrl_set_hackrf_one(gpo);
+	switchctrl_set_hackrf_one(rf_path, gpo);
 #else
 	(void)gpo;
 #endif
 }
 
-void rf_path_pin_setup() {
+void rf_path_pin_setup(rf_path_t* const rf_path) {
 #ifdef HACKRF_ONE
 	/* Configure RF switch control signals */
 	scu_pinmux(SCU_HP,             SCU_GPIO_FAST | SCU_CONF_FUNCTION0);
@@ -187,149 +187,154 @@ void rf_path_pin_setup() {
 	scu_pinmux(SCU_NO_VAA_ENABLE,  SCU_GPIO_FAST | SCU_CONF_FUNCTION0);
 
 	/* Configure RF switch control signals as outputs */
-	GPIO0_DIR |= PIN_AMP_BYPASS;
-	GPIO1_DIR |= (PIN_NO_MIX_BYPASS | PIN_RX_AMP | PIN_NO_RX_AMP_PWR);
-	GPIO2_DIR |= (PIN_HP | PIN_LP | PIN_TX_MIX_BP | PIN_RX_MIX_BP | PIN_TX_AMP);
-	GPIO3_DIR |= PIN_NO_TX_AMP_PWR;
-	GPIO5_DIR |= (PIN_TX | PIN_MIX_BYPASS | PIN_RX);
+	gpio_output(rf_path->gpio_amp_bypass);
+	gpio_output(rf_path->gpio_no_mix_bypass);
+	gpio_output(rf_path->gpio_rx_amp);
+	gpio_output(rf_path->gpio_no_rx_amp_pwr);
+	gpio_output(rf_path->gpio_hp);
+	gpio_output(rf_path->gpio_lp);
+	gpio_output(rf_path->gpio_tx_mix_bp);
+	gpio_output(rf_path->gpio_rx_mix_bp);
+	gpio_output(rf_path->gpio_tx_amp);
+	gpio_output(rf_path->gpio_no_tx_amp_pwr);
+	gpio_output(rf_path->gpio_tx);
+	gpio_output(rf_path->gpio_mix_bypass);
+	gpio_output(rf_path->gpio_rx);
 
 	/*
 	 * Safe (initial) switch settings turn off both amplifiers and antenna port
 	 * power and enable both amp bypass and mixer bypass.
 	 */
-	switchctrl_set(SWITCHCTRL_AMP_BYPASS | SWITCHCTRL_MIX_BYPASS);
-
-	/* Configure RF power supply (VAA) switch control signal as output */
-	GPIO_DIR(PORT_NO_VAA_ENABLE) |= PIN_NO_VAA_ENABLE;
-
-	/* Safe state: start with VAA turned off: */
-	disable_rf_power();
+	switchctrl_set(rf_path, SWITCHCTRL_AMP_BYPASS | SWITCHCTRL_MIX_BYPASS);
+#else
+	(void) rf_path; /* silence unused param warning */
 #endif
 }
 
-void rf_path_init(void) {
+void rf_path_init(rf_path_t* const rf_path) {
 	ssp1_set_mode_max5864();
-	max5864_shutdown();
+	max5864_setup(&max5864);
+	max5864_shutdown(&max5864);
 	
 	ssp1_set_mode_max2837();
-	max2837_setup();
-	max2837_start();
+	max2837_setup(&max2837);
+	max2837_start(&max2837);
 	
-	rffc5071_setup();
-	switchctrl_set(switchctrl);
+	rffc5071_setup(&rffc5072);
+	switchctrl_set(rf_path, switchctrl);
 }
 
-void rf_path_set_direction(const rf_path_direction_t direction) {
+void rf_path_set_direction(rf_path_t* const rf_path, const rf_path_direction_t direction) {
 	/* Turn off TX and RX amplifiers, then enable based on direction and bypass state. */
-	switchctrl |= SWITCHCTRL_NO_TX_AMP_PWR | SWITCHCTRL_NO_RX_AMP_PWR;
+	rf_path->switchctrl |= SWITCHCTRL_NO_TX_AMP_PWR | SWITCHCTRL_NO_RX_AMP_PWR;
 	switch(direction) {
 	case RF_PATH_DIRECTION_TX:
-		switchctrl |= SWITCHCTRL_TX;
-		if( (switchctrl & SWITCHCTRL_AMP_BYPASS) == 0 ) {
+		rf_path->switchctrl |= SWITCHCTRL_TX;
+		if( (rf_path->switchctrl & SWITCHCTRL_AMP_BYPASS) == 0 ) {
 			/* TX amplifier is in path, be sure to enable TX amplifier. */
-			switchctrl &= ~SWITCHCTRL_NO_TX_AMP_PWR;
+			rf_path->switchctrl &= ~SWITCHCTRL_NO_TX_AMP_PWR;
 		}
-		rffc5071_tx();
-		if( switchctrl & SWITCHCTRL_MIX_BYPASS ) {
-			rffc5071_disable();
+		rffc5071_tx(&rffc5072);
+		if( rf_path->switchctrl & SWITCHCTRL_MIX_BYPASS ) {
+			rffc5071_disable(&rffc5072);
 		} else {
-			rffc5071_enable();
+			rffc5071_enable(&rffc5072);
 		}
 		ssp1_set_mode_max5864();
-		max5864_tx();
+		max5864_tx(&max5864);
 		ssp1_set_mode_max2837();
-		max2837_tx();
-		sgpio_configure(SGPIO_DIRECTION_TX);
+		max2837_tx(&max2837);
+		sgpio_configure(&sgpio_config, SGPIO_DIRECTION_TX);
 		break;
 	
 	case RF_PATH_DIRECTION_RX:
-		switchctrl &= ~SWITCHCTRL_TX;
-		if( (switchctrl & SWITCHCTRL_AMP_BYPASS) == 0 ) {
+		rf_path->switchctrl &= ~SWITCHCTRL_TX;
+		if( (rf_path->switchctrl & SWITCHCTRL_AMP_BYPASS) == 0 ) {
 			/* RX amplifier is in path, be sure to enable RX amplifier. */
-			switchctrl &= ~SWITCHCTRL_NO_RX_AMP_PWR;
+			rf_path->switchctrl &= ~SWITCHCTRL_NO_RX_AMP_PWR;
 		}
-		rffc5071_rx();
-		if( switchctrl & SWITCHCTRL_MIX_BYPASS ) {
-			rffc5071_disable();
+		rffc5071_rx(&rffc5072);
+		if( rf_path->switchctrl & SWITCHCTRL_MIX_BYPASS ) {
+			rffc5071_disable(&rffc5072);
 		} else {
-			rffc5071_enable();
+			rffc5071_enable(&rffc5072);
 		}
 		ssp1_set_mode_max5864();
-		max5864_rx();
+		max5864_rx(&max5864);
 		ssp1_set_mode_max2837();
-		max2837_rx();
-		sgpio_configure(SGPIO_DIRECTION_RX);
+		max2837_rx(&max2837);
+		sgpio_configure(&sgpio_config, SGPIO_DIRECTION_RX);
 		break;
 		
 	case RF_PATH_DIRECTION_OFF:
 	default:
 #ifdef HACKRF_ONE
-		rf_path_set_antenna(0);
+		rf_path_set_antenna(rf_path, 0);
 #endif
-		rf_path_set_lna(0);
+		rf_path_set_lna(rf_path, 0);
 		/* Set RF path to receive direction when "off" */
-		switchctrl &= ~SWITCHCTRL_TX;
-		rffc5071_disable();
+		rf_path->switchctrl &= ~SWITCHCTRL_TX;
+		rffc5071_disable(&rffc5072);
 		ssp1_set_mode_max5864();
-		max5864_standby();
+		max5864_standby(&max5864);
 		ssp1_set_mode_max2837();
-		max2837_set_mode(MAX2837_MODE_STANDBY);
-		sgpio_configure(SGPIO_DIRECTION_RX);
+		max2837_set_mode(&max2837, MAX2837_MODE_STANDBY);
+		sgpio_configure(&sgpio_config, SGPIO_DIRECTION_RX);
 		break;
 	}
 
-	switchctrl_set(switchctrl);
+	switchctrl_set(rf_path, rf_path->switchctrl);
 }
 
-void rf_path_set_filter(const rf_path_filter_t filter) {
+void rf_path_set_filter(rf_path_t* const rf_path, const rf_path_filter_t filter) {
 	switch(filter) {
 	default:
 	case RF_PATH_FILTER_BYPASS:
-		switchctrl |= SWITCHCTRL_MIX_BYPASS;
-		rffc5071_disable();
+		rf_path->switchctrl |= SWITCHCTRL_MIX_BYPASS;
+		rffc5071_disable(&rffc5072);
 		break;
 		
 	case RF_PATH_FILTER_LOW_PASS:
-		switchctrl &= ~(SWITCHCTRL_HP | SWITCHCTRL_MIX_BYPASS);
-		rffc5071_enable();
+		rf_path->switchctrl &= ~(SWITCHCTRL_HP | SWITCHCTRL_MIX_BYPASS);
+		rffc5071_enable(&rffc5072);
 		break;
 		
 	case RF_PATH_FILTER_HIGH_PASS:
-		switchctrl &= ~SWITCHCTRL_MIX_BYPASS;
-		switchctrl |= SWITCHCTRL_HP;
-		rffc5071_enable();
+		rf_path->switchctrl &= ~SWITCHCTRL_MIX_BYPASS;
+		rf_path->switchctrl |= SWITCHCTRL_HP;
+		rffc5071_enable(&rffc5072);
 		break;
 	}
 
-	switchctrl_set(switchctrl);
+	switchctrl_set(rf_path, rf_path->switchctrl);
 }
 
-void rf_path_set_lna(const uint_fast8_t enable) {
+void rf_path_set_lna(rf_path_t* const rf_path, const uint_fast8_t enable) {
 	if( enable ) {
-		if( switchctrl & SWITCHCTRL_TX ) {
+		if( rf_path->switchctrl & SWITCHCTRL_TX ) {
 			/* AMP_BYPASS=0, NO_RX_AMP_PWR=1, NO_TX_AMP_PWR=0 */
-			switchctrl |= SWITCHCTRL_NO_RX_AMP_PWR;
-			switchctrl &= ~(SWITCHCTRL_AMP_BYPASS | SWITCHCTRL_NO_TX_AMP_PWR);
+			rf_path->switchctrl |= SWITCHCTRL_NO_RX_AMP_PWR;
+			rf_path->switchctrl &= ~(SWITCHCTRL_AMP_BYPASS | SWITCHCTRL_NO_TX_AMP_PWR);
 		} else {
 			/* AMP_BYPASS=0, NO_RX_AMP_PWR=0, NO_TX_AMP_PWR=1 */
-			switchctrl |= SWITCHCTRL_NO_TX_AMP_PWR;
-			switchctrl &= ~(SWITCHCTRL_AMP_BYPASS | SWITCHCTRL_NO_RX_AMP_PWR);
+			rf_path->switchctrl |= SWITCHCTRL_NO_TX_AMP_PWR;
+			rf_path->switchctrl &= ~(SWITCHCTRL_AMP_BYPASS | SWITCHCTRL_NO_RX_AMP_PWR);
 		}
 	} else {
 		/* AMP_BYPASS=1, NO_RX_AMP_PWR=1, NO_TX_AMP_PWR=1 */
-		switchctrl |= SWITCHCTRL_AMP_BYPASS | SWITCHCTRL_NO_TX_AMP_PWR | SWITCHCTRL_NO_RX_AMP_PWR;
+		rf_path->switchctrl |= SWITCHCTRL_AMP_BYPASS | SWITCHCTRL_NO_TX_AMP_PWR | SWITCHCTRL_NO_RX_AMP_PWR;
 	}
 	
-	switchctrl_set(switchctrl);
+	switchctrl_set(rf_path, rf_path->switchctrl);
 }
 
 /* antenna port power control */
-void rf_path_set_antenna(const uint_fast8_t enable) {
+void rf_path_set_antenna(rf_path_t* const rf_path, const uint_fast8_t enable) {
 	if (enable) {
-		switchctrl |= SWITCHCTRL_ANT_PWR;
+		rf_path->switchctrl |= SWITCHCTRL_ANT_PWR;
 	} else {
-		switchctrl &= ~(SWITCHCTRL_ANT_PWR);
+		rf_path->switchctrl &= ~(SWITCHCTRL_ANT_PWR);
 	}
 
-	switchctrl_set(switchctrl);
+	switchctrl_set(rf_path, rf_path->switchctrl);
 }
