@@ -26,14 +26,17 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-int main(int argc, char** argv)
+int main(void)
 {
 	int result = HACKRF_SUCCESS;
 	uint8_t board_id = BOARD_ID_INVALID;
 	char version[255 + 1];
+	uint16_t usb_version;
 	read_partid_serialno_t read_partid_serialno;
+	uint8_t operacakes[8];
 	hackrf_device_list_t *list;
-	int i;
+	hackrf_device* device;
+	int i, j;
 
 	result = hackrf_init();
 	if (result != HACKRF_SUCCESS) {
@@ -41,6 +44,10 @@ int main(int argc, char** argv)
 				hackrf_error_name(result), result);
 		return EXIT_FAILURE;
 	}
+
+	printf("hackrf_info version: %s\n", TOOL_RELEASE);
+	printf("libhackrf version: %s (%s)\n", hackrf_library_release(),
+			hackrf_library_version());
 	
 	list = hackrf_device_list();
 	
@@ -53,12 +60,13 @@ int main(int argc, char** argv)
 		if (i > 0)
 			printf("\n");
 			
-		printf("Found HackRF board %d:\n", i);
+		printf("Found HackRF\n");
+		printf("Index: %d\n", i);
 		
 		if (list->serial_numbers[i])
-			printf("USB descriptor string: %s\n", list->serial_numbers[i]);
+			printf("Serial number: %s\n", list->serial_numbers[i]);
 
-		hackrf_device* device = NULL;
+		device = NULL;
 		result = hackrf_device_list_open(list, i, &device);
 		if (result != HACKRF_SUCCESS) {
 			fprintf(stderr, "hackrf_open() failed: %s (%d)\n",
@@ -81,7 +89,15 @@ int main(int argc, char** argv)
 					hackrf_error_name(result), result);
 			return EXIT_FAILURE;
 		}
-		printf("Firmware Version: %s\n", version);
+
+		result = hackrf_usb_api_version_read(device, &usb_version);
+		if (result != HACKRF_SUCCESS) {
+			fprintf(stderr, "hackrf_usb_api_version_read() failed: %s (%d)\n",
+					hackrf_error_name(result), result);
+			return EXIT_FAILURE;
+		}
+		printf("Firmware Version: %s (API:%x.%02x)\n", version,
+				(usb_version>>8)&0xFF, usb_version&0xFF);
 
 		result = hackrf_board_partid_serialno_read(device, &read_partid_serialno);	
 		if (result != HACKRF_SUCCESS) {
@@ -92,12 +108,21 @@ int main(int argc, char** argv)
 		printf("Part ID Number: 0x%08x 0x%08x\n", 
 					read_partid_serialno.part_id[0],
 					read_partid_serialno.part_id[1]);
-		printf("Serial Number: 0x%08x 0x%08x 0x%08x 0x%08x\n", 
-					read_partid_serialno.serial_no[0],
-					read_partid_serialno.serial_no[1],
-					read_partid_serialno.serial_no[2],
-					read_partid_serialno.serial_no[3]);
-		
+
+		result = hackrf_get_operacake_boards(device, &operacakes[0]);
+		if ((result != HACKRF_SUCCESS) && (result != HACKRF_ERROR_USB_API_VERSION)) {
+			fprintf(stderr, "hackrf_get_operacake_boards() failed: %s (%d)\n",
+					hackrf_error_name(result), result);
+				return EXIT_FAILURE;
+		}
+		if(result == HACKRF_SUCCESS) {
+			for(j=0; j<8; j++) {
+				if(operacakes[j] == 0)
+					break;
+				printf("Operacake found, address: 0x%02x\n", operacakes[j]);
+			}
+		}
+
 		result = hackrf_close(device);
 		if (result != HACKRF_SUCCESS) {
 			fprintf(stderr, "hackrf_close() failed: %s (%d)\n",
