@@ -24,8 +24,13 @@ ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSI
 #include "hackrf.h"
 
 #include <stdlib.h>
-
+#include <string.h>
 #include <libusb.h>
+
+#ifdef _WIN32
+/* Avoid redefinition of timespec from time.h (included by libusb.h) */
+#define HAVE_STRUCT_TIMESPEC 1
+#endif
 #include <pthread.h>
 
 #ifndef bool
@@ -67,6 +72,12 @@ typedef enum {
 	HACKRF_VENDOR_REQUEST_SET_TXVGA_GAIN = 21,
 	HACKRF_VENDOR_REQUEST_ANTENNA_ENABLE = 23,
 	HACKRF_VENDOR_REQUEST_SET_FREQ_EXPLICIT = 24,
+	// USB_WCID_VENDOR_REQ = 25
+	HACKRF_VENDOR_REQUEST_INIT_SWEEP = 26,
+	HACKRF_VENDOR_REQUEST_OPERACAKE_GET_BOARDS = 27,
+	HACKRF_VENDOR_REQUEST_OPERACAKE_SET_PORTS = 28,
+	HACKRF_VENDOR_REQUEST_SET_HW_SYNC_MODE = 29,
+	HACKRF_VENDOR_REQUEST_RESET = 30,
 } hackrf_vendor_request;
 
 typedef enum {
@@ -78,19 +89,28 @@ typedef enum {
 	HACKRF_TRANSCEIVER_MODE_OFF = 0,
 	HACKRF_TRANSCEIVER_MODE_RECEIVE = 1,
 	HACKRF_TRANSCEIVER_MODE_TRANSMIT = 2,
+	HACKRF_TRANSCEIVER_MODE_SS = 3,
+	TRANSCEIVER_MODE_CPLD_UPDATE = 4,
 } hackrf_transceiver_mode;
 
+typedef enum {
+	HACKRF_HW_SYNC_MODE_OFF = 0,
+	HACKRF_HW_SYNC_MODE_ON = 1,
+} hackrf_hw_sync_mode;
+
+#define TRANSFER_COUNT 4
+#define TRANSFER_BUFFER_SIZE 262144
+
 struct hackrf_device {
 	libusb_device_handle* usb_device;
 	struct libusb_transfer** transfers;
 	hackrf_sample_block_cb_fn callback;
 	volatile bool transfer_thread_started; /* volatile shared between threads (read only) */
 	pthread_t transfer_thread;
-	uint32_t transfer_count;
-	uint32_t buffer_size;
 	volatile bool streaming; /* volatile shared between threads (read only) */
 	void* rx_ctx;
 	void* tx_ctx;
+	unsigned char buffer[TRANSFER_COUNT * TRANSFER_BUFFER_SIZE];
 };
 
 typedef struct {
@@ -117,7 +137,7 @@ static const max2837_ft_t max2837_ft[] = {
 	{ 0        }
 };
 
-volatile bool do_exit = false;
+static volatile bool do_exit = false;
 
 static const uint16_t hackrf_usb_vid = 0x1d50;
 static const uint16_t hackrf_jawbreaker_usb_pid = 0x604b;
@@ -125,6 +145,7 @@ static const uint16_t hackrf_one_usb_pid = 0x6089;
 static const uint16_t rad1o_usb_pid = 0xcc15;
 
 static libusb_context* g_libusb_context = NULL;
+int last_libusb_error = LIBUSB_SUCCESS;
 
 static void request_exit(void)
 {
@@ -137,7 +158,7 @@ static int cancel_transfers(hackrf_device* device)
 
 	if( device->transfers != NULL )
 	{
-		for(transfer_index=0; transfer_index<device->transfer_count; transfer_index++)
+		for(transfer_index=0; transfer_index<TRANSFER_COUNT; transfer_index++)
 		{
 			if( device->transfers[transfer_index] != NULL )
 			{
@@ -157,7 +178,7 @@ static int free_transfers(hackrf_device* device)
 	if( device->transfers != NULL )
 	{
 		// libusb_close() should free all transfers referenced from this array.
-		for(transfer_index=0; transfer_index<device->transfer_count; transfer_index++)
+		for(transfer_index=0; transfer_index<TRANSFER_COUNT; transfer_index++)
 		{
 			if( device->transfers[transfer_index] != NULL )
 			{
@@ -176,13 +197,13 @@ static int allocate_transfers(hackrf_device* const device)
 	if( device->transfers == NULL )
 	{
 		uint32_t transfer_index;
-		device->transfers = (struct libusb_transfer**) calloc(device->transfer_count, sizeof(struct libusb_transfer));
+		device->transfers = (struct libusb_transfer**) calloc(TRANSFER_COUNT, sizeof(struct libusb_transfer));
 		if( device->transfers == NULL )
 		{
 			return HACKRF_ERROR_NO_MEM;
 		}
 
-		for(transfer_index=0; transfer_index<device->transfer_count; transfer_index++)
+		for(transfer_index=0; transfer_index<TRANSFER_COUNT; transfer_index++)
 		{
 			device->transfers[transfer_index] = libusb_alloc_transfer(0);
 			if( device->transfers[transfer_index] == NULL )
@@ -194,8 +215,8 @@ static int allocate_transfers(hackrf_device* const device)
 				device->transfers[transfer_index],
 				device->usb_device,
 				0,
-				(unsigned char*)malloc(device->buffer_size),
-				device->buffer_size,
+				&device->buffer[transfer_index * TRANSFER_BUFFER_SIZE],
+				TRANSFER_BUFFER_SIZE,
 				NULL,
 				device,
 				0
@@ -221,7 +242,7 @@ static int prepare_transfers(
 	uint32_t transfer_index;
 	if( device->transfers != NULL )
 	{
-		for(transfer_index=0; transfer_index<device->transfer_count; transfer_index++)
+		for(transfer_index=0; transfer_index<TRANSFER_COUNT; transfer_index++)
 		{
 			device->transfers[transfer_index]->endpoint = endpoint_address;
 			device->transfers[transfer_index]->callback = callback;
@@ -229,6 +250,7 @@ static int prepare_transfers(
 			error = libusb_submit_transfer(device->transfers[transfer_index]);
 			if( error != 0 )
 			{
+				last_libusb_error = error;
 				return HACKRF_ERROR_LIBUSB;
 			}
 		}
@@ -249,6 +271,7 @@ static int detach_kernel_drivers(libusb_device_handle* usb_device_handle)
 	result = libusb_get_active_config_descriptor(dev, &config);
 	if( result < 0 )
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	}
 
@@ -262,11 +285,13 @@ static int detach_kernel_drivers(libusb_device_handle* usb_device_handle)
 			if( result == LIBUSB_ERROR_NOT_SUPPORTED ) {
 				return 0;
 			}
+			last_libusb_error = result;
 			return HACKRF_ERROR_LIBUSB;
 		} else if( result == 1 ) {
 			result = libusb_detach_kernel_driver(usb_device_handle, i);
 			if( result != 0 )
 			{
+				last_libusb_error = result;
 				return HACKRF_ERROR_LIBUSB;
 			}
 		}
@@ -280,6 +305,7 @@ static int set_hackrf_configuration(libusb_device_handle* usb_device, int config
 	result = libusb_get_configuration(usb_device, &curr_config);
 	if( result != 0 )
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	}
 
@@ -293,6 +319,7 @@ static int set_hackrf_configuration(libusb_device_handle* usb_device, int config
 		result = libusb_set_configuration(usb_device, config);
 		if( result != 0 )
 		{
+			last_libusb_error = result;
 			return HACKRF_ERROR_LIBUSB;
 		}
 	}
@@ -312,13 +339,15 @@ extern "C"
 
 int ADDCALL hackrf_init(void)
 {
+	int libusb_error;
 	if (g_libusb_context != NULL) {
 		return HACKRF_SUCCESS;
 	}
 	
-	const int libusb_error = libusb_init(&g_libusb_context);
+	libusb_error = libusb_init(&g_libusb_context);
 	if( libusb_error != 0 )
 	{
+		last_libusb_error = libusb_error;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -336,13 +365,30 @@ int ADDCALL hackrf_exit(void)
 	return HACKRF_SUCCESS;
 }
 
-#include <stdio.h>
-#include <string.h>
+#ifndef LIBRARY_VERSION
+#define LIBRARY_VERSION "unknown"
+#endif
+const char* ADDCALL hackrf_library_version()
+{
+	return LIBRARY_VERSION;
+}
+
+#ifndef LIBRARY_RELEASE
+#define LIBRARY_RELEASE "unknown"
+#endif
+const char* ADDCALL hackrf_library_release()
+{
+	return LIBRARY_RELEASE;
+}
 
 hackrf_device_list_t* ADDCALL hackrf_device_list()
 {
 	ssize_t i;
 	libusb_device_handle* usb_device = NULL;
+	uint_fast8_t serial_descriptor_index;
+	char serial_number[64];
+	int serial_number_length;
+
 	hackrf_device_list_t* list = calloc(1, sizeof(*list));
 	if ( list == NULL )
 		return NULL;
@@ -370,14 +416,13 @@ hackrf_device_list_t* ADDCALL hackrf_device_list()
 				list->usb_board_ids[idx] = device_descriptor.idProduct;
 				list->usb_device_index[idx] = i;
 				
-				const uint_fast8_t serial_descriptor_index = device_descriptor.iSerialNumber;
+				serial_descriptor_index = device_descriptor.iSerialNumber;
 				if( serial_descriptor_index > 0 ) {
 					if( libusb_open(list->usb_devices[i], &usb_device) != 0 ) {
 						usb_device = NULL;
 						continue;
 					}
-					char serial_number[64];
-					const int serial_number_length = libusb_get_string_descriptor_ascii(usb_device, serial_descriptor_index, (unsigned char*)serial_number, sizeof(serial_number));
+					serial_number_length = libusb_get_string_descriptor_ascii(usb_device, serial_descriptor_index, (unsigned char*)serial_number, sizeof(serial_number));
 					if( serial_number_length == 32 ) {
 						serial_number[32] = 0;
 						list->serial_numbers[idx] = strdup(serial_number);
@@ -417,8 +462,8 @@ libusb_device_handle* hackrf_open_usb(const char* const desired_serial_number)
 	const ssize_t list_length = libusb_get_device_list(g_libusb_context, &devices);
 	int match_len = 0;
 	ssize_t i;
-	
-	printf("Number of USB devices: %ld\n", list_length);
+	char serial_number[64];
+	int serial_number_length;
 	
 	if( desired_serial_number ) {
 		/* If a shorter serial number is specified, only match against the suffix.
@@ -437,7 +482,6 @@ libusb_device_handle* hackrf_open_usb(const char* const desired_serial_number)
 			if((device_descriptor.idProduct == hackrf_one_usb_pid) ||
 			   (device_descriptor.idProduct == hackrf_jawbreaker_usb_pid) ||
 			   (device_descriptor.idProduct == rad1o_usb_pid)) {
-				printf("USB device %4x:%4x:", device_descriptor.idVendor, device_descriptor.idProduct);
 				
 				if( desired_serial_number != NULL ) {
 					const uint_fast8_t serial_descriptor_index = device_descriptor.iSerialNumber;
@@ -446,27 +490,21 @@ libusb_device_handle* hackrf_open_usb(const char* const desired_serial_number)
 							usb_device = NULL;
 							continue;
 						}
-						char serial_number[64];
-						const int serial_number_length = libusb_get_string_descriptor_ascii(usb_device, serial_descriptor_index, (unsigned char*)serial_number, sizeof(serial_number));
+						serial_number_length = libusb_get_string_descriptor_ascii(usb_device, serial_descriptor_index, (unsigned char*)serial_number, sizeof(serial_number));
 						if( serial_number_length == 32 ) {
 							serial_number[32] = 0;
-							printf(" %s", serial_number);
 							if( strncmp(serial_number + 32-match_len, desired_serial_number, match_len) == 0 ) {
-								printf(" match\n");
 								break;
 							} else {
-								printf(" skip\n");
 								libusb_close(usb_device);
 								usb_device = NULL;
 							}
 						} else {
-							printf(" wrong length of serial number: %d\n", serial_number_length);
 							libusb_close(usb_device);
 							usb_device = NULL;
 						}
 					}
 				} else {
-					printf(" default\n");
 					libusb_open(devices[i], &usb_device);
 					break;
 				}
@@ -497,6 +535,7 @@ static int hackrf_open_setup(libusb_device_handle* usb_device, hackrf_device** d
 	result = libusb_claim_interface(usb_device, 0);
 	if( result != LIBUSB_SUCCESS )
 	{
+		last_libusb_error = result;
 		libusb_close(usb_device);
 		return HACKRF_ERROR_LIBUSB;
 	}
@@ -514,12 +553,6 @@ static int hackrf_open_setup(libusb_device_handle* usb_device, hackrf_device** d
 	lib_device->transfers = NULL;
 	lib_device->callback = NULL;
 	lib_device->transfer_thread_started = false;
-	/*
-	lib_device->transfer_count = 1024;
-	lib_device->buffer_size = 16384;
-	*/
-	lib_device->transfer_count = 4;
-	lib_device->buffer_size = 262144; /* 1048576; */
 	lib_device->streaming = false;
 	do_exit = false;
 
@@ -593,16 +626,19 @@ int ADDCALL hackrf_open_by_serial(const char* const desired_serial_number, hackr
 int ADDCALL hackrf_device_list_open(hackrf_device_list_t *list, int idx, hackrf_device** device)
 {
 	libusb_device_handle* usb_device;
+	int i, result;
 	
 	if( device == NULL || list == NULL || idx < 0 || idx >= list->devicecount )
 	{
 		return HACKRF_ERROR_INVALID_PARAM;
 	}
 	
-	int i = list->usb_device_index[idx];
+	i = list->usb_device_index[idx];
 
-	if( libusb_open(list->usb_devices[i], &usb_device) != 0 ) {
+	result = libusb_open(list->usb_devices[i], &usb_device);
+	if(result != 0) {
 		usb_device = NULL;
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	}
 	
@@ -625,6 +661,7 @@ int ADDCALL hackrf_set_transceiver_mode(hackrf_device* device, hackrf_transceive
 
 	if( result != 0 )
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -653,6 +690,7 @@ int ADDCALL hackrf_max2837_read(hackrf_device* device, uint8_t register_number,
 
 	if( result < 2 )
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -685,6 +723,7 @@ int ADDCALL hackrf_max2837_write(hackrf_device* device, uint8_t register_number,
 
 	if( result != 0 )
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -715,6 +754,7 @@ int ADDCALL hackrf_si5351c_read(hackrf_device* device, uint16_t register_number,
 
 	if( result < 1 )
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		*value = temp_value;
@@ -747,6 +787,7 @@ int ADDCALL hackrf_si5351c_write(hackrf_device* device, uint16_t register_number
 
 	if( result != 0 )
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -769,6 +810,7 @@ int ADDCALL hackrf_set_baseband_filter_bandwidth(hackrf_device* device, const ui
 
 	if( result != 0 )
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -798,6 +840,7 @@ int ADDCALL hackrf_rffc5071_read(hackrf_device* device, uint8_t register_number,
 
 	if( result < 2 )
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -826,6 +869,7 @@ int ADDCALL hackrf_rffc5071_write(hackrf_device* device, uint8_t register_number
 
 	if( result != 0 )
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -848,6 +892,7 @@ int ADDCALL hackrf_spiflash_erase(hackrf_device* device)
 
 	if (result != 0)
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -877,6 +922,7 @@ int ADDCALL hackrf_spiflash_write(hackrf_device* device, const uint32_t address,
 
 	if (result < length)
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -906,6 +952,7 @@ int ADDCALL hackrf_spiflash_read(hackrf_device* device, const uint32_t address,
 
 	if (result < length)
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -935,6 +982,7 @@ int ADDCALL hackrf_cpld_write(hackrf_device* device,
 		);
 
 		if (result != LIBUSB_SUCCESS) {
+			last_libusb_error = result;
 			return HACKRF_ERROR_LIBUSB;
 		}
 	}
@@ -958,6 +1006,7 @@ int ADDCALL hackrf_board_id_read(hackrf_device* device, uint8_t* value)
 
 	if (result < 1)
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -981,6 +1030,7 @@ int ADDCALL hackrf_version_string_read(hackrf_device* device, char* version,
 
 	if (result < 0)
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		version[result] = '\0';
@@ -988,6 +1038,29 @@ int ADDCALL hackrf_version_string_read(hackrf_device* device, char* version,
 	}
 }
 
+extern ADDAPI int ADDCALL hackrf_usb_api_version_read(hackrf_device* device,
+		uint16_t* version)
+{
+	int result;
+	libusb_device* dev;
+	struct libusb_device_descriptor desc;
+	dev = libusb_get_device(device->usb_device);
+	result = libusb_get_device_descriptor(dev, &desc);
+	if (result < 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	}
+
+	*version = desc.bcdDevice;
+	return HACKRF_SUCCESS;
+}
+
+#define USB_API_REQUIRED(device, version)          \
+uint16_t usb_version = 0;                          \
+hackrf_usb_api_version_read(device, &usb_version); \
+if(usb_version < version)                          \
+	return HACKRF_ERROR_USB_API_VERSION;
+
 typedef struct {
 	uint32_t freq_mhz; /* From 0 to 6000+MHz */
 	uint32_t freq_hz; /* From 0 to 999999Hz */
@@ -1023,6 +1096,7 @@ int ADDCALL hackrf_set_freq(hackrf_device* device, const uint64_t freq_hz)
 
 	if (result < length)
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -1074,6 +1148,7 @@ int ADDCALL hackrf_set_freq_explicit(hackrf_device* device,
 
 	if (result < length)
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -1086,6 +1161,11 @@ typedef struct {
 } set_fracrate_params_t;
 
 
+/*
+ * You should probably use hackrf_set_sample_rate() below instead of this
+ * function.  They both result in automatic baseband filter selection as
+ * described below.
+ */
 int ADDCALL hackrf_set_sample_rate_manual(hackrf_device* device,
                                        const uint32_t freq_hz, uint32_t divider)
 {
@@ -1110,12 +1190,20 @@ int ADDCALL hackrf_set_sample_rate_manual(hackrf_device* device,
 
 	if (result < length)
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
-		return HACKRF_SUCCESS;
+		return hackrf_set_baseband_filter_bandwidth(device,
+				hackrf_compute_baseband_filter_bw((uint32_t)(0.75*freq_hz/divider)));
 	}
 }
 
+/*
+ * For anti-aliasing, the baseband filter bandwidth is automatically set to the
+ * widest available setting that is no more than 75% of the sample rate.  This
+ * happens every time the sample rate is set.  If you want to override the
+ * baseband filter selection, you must do so after setting the sample rate.
+ */
 int ADDCALL hackrf_set_sample_rate(hackrf_device* device, const double freq)
 {
 	const int MAX_N = 32;
@@ -1172,6 +1260,7 @@ int ADDCALL hackrf_set_amp_enable(hackrf_device* device, const uint8_t value)
 
 	if (result != 0)
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -1197,6 +1286,7 @@ int ADDCALL hackrf_board_partid_serialno_read(hackrf_device* device, read_partid
 
 	if (result < length)
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 
@@ -1316,6 +1406,7 @@ int ADDCALL hackrf_set_antenna_enable(hackrf_device* device, const uint8_t value
 
 	if (result != 0)
 	{
+		last_libusb_error = result;
 		return HACKRF_ERROR_LIBUSB;
 	} else {
 		return HACKRF_SUCCESS;
@@ -1347,12 +1438,12 @@ static void hackrf_libusb_transfer_callback(struct libusb_transfer* usb_transfer
 	if(usb_transfer->status == LIBUSB_TRANSFER_COMPLETED)
 	{
 		hackrf_transfer transfer = {
-			transfer.device = device,
-			transfer.buffer = usb_transfer->buffer,
-			transfer.buffer_length = usb_transfer->length,
-			transfer.valid_length = usb_transfer->actual_length,
-			transfer.rx_ctx = device->rx_ctx,
-			transfer.tx_ctx = device->tx_ctx
+			.device = device,
+			.buffer = usb_transfer->buffer,
+			.buffer_length = usb_transfer->length,
+			.valid_length = usb_transfer->actual_length,
+			.rx_ctx = device->rx_ctx,
+			.tx_ctx = device->tx_ctx
 		};
 
 		if( device->callback(&transfer) == 0 )
@@ -1409,6 +1500,7 @@ static int create_transfer_thread(hackrf_device* device,
 	if( device->transfer_thread_started == false )
 	{
 		device->streaming = false;
+		do_exit = false;
 
 		result = prepare_transfers(
 			device, endpoint_address,
@@ -1551,37 +1643,44 @@ const char* ADDCALL hackrf_error_name(enum hackrf_error errcode)
 		return "HACKRF_TRUE";
 
 	case HACKRF_ERROR_INVALID_PARAM:
-		return "HACKRF_ERROR_INVALID_PARAM";
+		return "invalid parameter(s)";
 
 	case HACKRF_ERROR_NOT_FOUND:
-		return "HACKRF_ERROR_NOT_FOUND";
+		return "HackRF not found";
 
 	case HACKRF_ERROR_BUSY:
-		return "HACKRF_ERROR_BUSY";
+		return "HackRF busy";
 
 	case HACKRF_ERROR_NO_MEM:
-		return "HACKRF_ERROR_NO_MEM";
+		return "insufficient memory";
 
 	case HACKRF_ERROR_LIBUSB:
-		return "HACKRF_ERROR_LIBUSB";
+#if defined(LIBUSB_API_VERSION) && (LIBUSB_API_VERSION >= 0x01000103)
+		if(last_libusb_error != LIBUSB_SUCCESS)
+			return libusb_strerror(last_libusb_error);
+#endif
+		return "USB error";
 
 	case HACKRF_ERROR_THREAD:
-		return "HACKRF_ERROR_THREAD";
+		return "transfer thread error";
 
 	case HACKRF_ERROR_STREAMING_THREAD_ERR:
-		return "HACKRF_ERROR_STREAMING_THREAD_ERR";
+		return "streaming thread encountered an error";
 
 	case HACKRF_ERROR_STREAMING_STOPPED:
-		return "HACKRF_ERROR_STREAMING_STOPPED";
+		return "streaming stopped";
 
 	case HACKRF_ERROR_STREAMING_EXIT_CALLED:
-		return "HACKRF_ERROR_STREAMING_EXIT_CALLED";
+		return "streaming terminated";
+
+	case HACKRF_ERROR_USB_API_VERSION:
+		return "feature not supported by installed firmware";
 
 	case HACKRF_ERROR_OTHER:
-		return "HACKRF_ERROR_OTHER";
+		return "unspecified error";
 
 	default:
-		return "HACKRF unknown error";
+		return "unknown error code";
 	}
 }
 
@@ -1598,6 +1697,9 @@ const char* ADDCALL hackrf_board_id_name(enum hackrf_board_id board_id)
 	case BOARD_ID_HACKRF_ONE:
 		return "HackRF One";
 
+	case BOARD_ID_RAD1O:
+		return "rad1o";
+
 	case BOARD_ID_INVALID:
 		return "Invalid Board ID";
 
@@ -1616,6 +1718,9 @@ extern ADDAPI const char* ADDCALL hackrf_usb_board_id_name(enum hackrf_usb_board
 	case USB_BOARD_ID_HACKRF_ONE:
 		return "HackRF One";
 
+	case USB_BOARD_ID_RAD1O:
+		return "rad1o";
+
 	case USB_BOARD_ID_INVALID:
 		return "Invalid Board ID";
 
@@ -1681,6 +1786,190 @@ uint32_t ADDCALL hackrf_compute_baseband_filter_bw(const uint32_t bandwidth_hz)
 	return p->bandwidth_hz;
 }
 
+/* All features below require USB API version 0x1002 or higher) */
+
+int ADDCALL hackrf_set_hw_sync_mode(hackrf_device* device, const uint8_t value) {
+	USB_API_REQUIRED(device, 0x0102)
+	int result = libusb_control_transfer(
+		device->usb_device,
+ 		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_SET_HW_SYNC_MODE,
+		value,
+		0,
+		NULL,
+		0,
+		0
+	);
+
+	if( result != 0 )
+	{
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+/*
+ * Initialize sweep mode:
+ * frequency_list is a list of start/stop pairs of frequencies in MHz.
+ * num_ranges is the number of pairs in frequency_list (1 to 10)
+ * num_samples is the number of samples to capture after each tuning.
+ * step_width is the width in Hz of the tuning step.
+ * offset is a number of Hz added to every tuning frequency.
+ *     Use to select center frequency based on the expected usable bandwidth.
+ * sweep_mode
+ *     LINEAR means step_width is added to the current frequency at each step.
+ *     INTERLEAVED invokes a scheme in which each step is divided into two
+ *         interleaved sub-steps, allowing the host to select the best portions
+ *         of the FFT of each sub-step and discard the rest.
+ */
+int ADDCALL hackrf_init_sweep(hackrf_device* device,
+		const uint16_t* frequency_list, const int num_ranges,
+		const uint32_t num_samples, const uint32_t step_width,
+		const uint32_t offset, const enum sweep_style style) {
+	USB_API_REQUIRED(device, 0x0102)
+	int result, i;
+	unsigned char data[9 + MAX_SWEEP_RANGES * 2 * sizeof(frequency_list[0])];
+	int size = 9 + num_ranges * 2 * sizeof(frequency_list[0]);
+
+	if((num_ranges < 1) || (num_ranges > MAX_SWEEP_RANGES)){
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+
+	if(num_samples % SAMPLES_PER_BLOCK) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+
+	if(SAMPLES_PER_BLOCK > num_samples) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+
+	if(1 > step_width) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+
+	if(INTERLEAVED < style) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+
+	data[0] = step_width & 0xff;
+	data[1] = (step_width >> 8) & 0xff;
+	data[2] = (step_width >> 16) & 0xff;
+	data[3] = (step_width >> 24) & 0xff;
+	data[4] = offset & 0xff;
+	data[5] = (offset >> 8) & 0xff;
+	data[6] = (offset >> 16) & 0xff;
+	data[7] = (offset >> 24) & 0xff;
+	data[8] = style;
+	for(i=0; i<(num_ranges*2); i++) {
+		data[9+i*2] = frequency_list[i] & 0xff;
+		data[10+i*2] = (frequency_list[i] >> 8) & 0xff;
+	}
+
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_INIT_SWEEP,
+		num_samples & 0xffff,
+		(num_samples >> 16) & 0xffff,
+		data,
+		size,
+		0
+	);
+
+	if (result < size) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+/* Retrieve list of Operacake board addresses 
+ * boards must be *uint8_t[8]
+ */
+int ADDCALL hackrf_get_operacake_boards(hackrf_device* device, uint8_t* boards)
+{
+	USB_API_REQUIRED(device, 0x0102)
+	int result;
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_IN | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_OPERACAKE_GET_BOARDS,
+		0,
+		0,
+		boards,
+		8,
+		0
+	);
+
+	if (result < 8)
+	{
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+/* Set Operacake ports */
+int ADDCALL hackrf_set_operacake_ports(hackrf_device* device,
+                                       uint8_t address,
+                                       uint8_t port_a,
+                                       uint8_t port_b)
+{
+	USB_API_REQUIRED(device, 0x0102)
+	int result;
+	/* Error checking */
+	if((port_a > OPERACAKE_PB4) || (port_b > OPERACAKE_PB4)) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+	/* Check which side PA and PB are on */
+	if(((port_a <= OPERACAKE_PA4) && (port_b <= OPERACAKE_PA4))
+	    || ((port_a > OPERACAKE_PA4) && (port_b > OPERACAKE_PA4))) {
+		return HACKRF_ERROR_INVALID_PARAM;
+	}
+	result = libusb_control_transfer(
+		device->usb_device,
+		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_OPERACAKE_SET_PORTS,
+		address,
+		port_a | (port_b<<8),
+		NULL,
+		0,
+		0
+	);
+
+	if (result != 0) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
+int ADDCALL hackrf_reset(hackrf_device* device) {
+	USB_API_REQUIRED(device, 0x0102)
+	int result = libusb_control_transfer(
+		device->usb_device,
+ 		LIBUSB_ENDPOINT_OUT | LIBUSB_REQUEST_TYPE_VENDOR | LIBUSB_RECIPIENT_DEVICE,
+		HACKRF_VENDOR_REQUEST_RESET,
+		0,
+		0,
+		NULL,
+		0,
+		0
+	);
+
+	if( result != 0 ) {
+		last_libusb_error = result;
+		return HACKRF_ERROR_LIBUSB;
+	} else {
+		return HACKRF_SUCCESS;
+	}
+}
+
 #ifdef __cplusplus
 } // __cplusplus defined.
 #endif
